﻿using System.IO;
using System.Web;
using System.Web.Mvc;
using KellermanSoftware.NetLinkTracker;
using KellermanSoftware.NetLinkTracker.Entities;
using NetLinkTrackerDemoMVC.Models;

namespace NetLinkTrackerDemoMVC.Controllers
{
    public class HomeController : Controller
    {
        public ActionResult Index()
        {
            return View();
        }

        public ActionResult ReviewLinks()
        {
            DataLayer.CheckSetup();
            LinkLogic linkLogic = new LinkLogic();
            ReviewLinksViewModel model = new ReviewLinksViewModel();
            model.TrackedLinks = linkLogic.DatabaseLogic.GetAllTrackedLinks();
            return View(model);
        }

        public ActionResult CreateLink()
        {
            ViewBag.Message = "Create Link";

            return View();
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult CreateLink(CreateLinkViewModel model)
        {
            if (ModelState.IsValid)
            {
                DataLayer.CheckSetup();
                LinkLogic linkLogic = new LinkLogic();

                TrackedLink trackedLink = linkLogic.CreateExpiringUrl(model.Url, model.ExpirationDays);
                model.CreatedLink = trackedLink.LinkGuid;
            }
            return View(model);
        }

        public ActionResult LinkHandler(string linkId)
        {
            ViewBag.Message = "Link Handler";
            LinkHandlerViewModel model = new LinkHandlerViewModel();

            LinkLogic linkLogic = new LinkLogic();
            TrackedLink trackedLink = linkLogic.GetLinkByGuid(linkId);

            LinkStatus status = linkLogic.CheckLinkStatus(trackedLink);

            model.Status = status.ToString();
            model.FriendlyStatus = linkLogic.GetFriendlyStatus(trackedLink);

            if (status == LinkStatus.Active)
                linkLogic.ProcessLink(trackedLink);

            return View(model);
        }

        [HttpGet]
        public ActionResult CreateDownloadLink()
        {
            ViewBag.Message = "Create Download Link";

            return View();
        }

        [HttpPost]
        public ActionResult CreateDownloadLink(CreateDownloadLinkViewModel model)
        {
            UploadFile(model);
            ValidateUpload(model);

            if (ModelState.IsValid)
            {
                string filePath = SaveFile(model);
                DataLayer.CheckSetup();
                LinkLogic linkLogic = new LinkLogic();
                TrackedLink trackedLink = linkLogic.CreateExpiringDownload(filePath, model.ExpirationDays);
                model.CreatedLink = trackedLink.LinkGuid;
            }

            return View(model);
        }

        private string SaveFile(CreateDownloadLinkViewModel model)
        {
            string rootPath = Server.MapPath("~");
            string downloadPath = Path.Combine(rootPath, "Downloads");
            if (!Directory.Exists(downloadPath))
                Directory.CreateDirectory(downloadPath);

            string fileName = Path.GetFileName(model.FileName);
            string filePath = Path.Combine(downloadPath, fileName);

            System.IO.File.WriteAllBytes(filePath, model.Data);

            return filePath;
        }

        private void ValidateUpload(CreateDownloadLinkViewModel model)
        {
            if (model.Data == null || model.Data.Length == 0)
            {
                ModelState.AddModelError("Data", "File is required");
            }
        }

        private void UploadFile(CreateDownloadLinkViewModel model)
        {
            HttpPostedFileBase file = Request.Files["FileData"];
            if (file != null && file.ContentLength > 0)
            {
                model.FileName = file.FileName;

                using (MemoryStream outStream = new MemoryStream())
                {
                    file.InputStream.CopyTo(outStream);
                    outStream.Seek(0, SeekOrigin.Begin);
                    model.Data = outStream.ToArray();
                }
            }
        }
    }
}